--mysql -hia64g -uensadmin -pensembl < efg_test.sql

--- Summary of current proposed/absent tables:
---     seq_region tables (incorporating chromosome name and assembly version)
---     xref tables - for linking probes to user/vendor defined features.
---     Denormalised probe_set/probe/probe_feature/results tables bt vendor, format or design_type? 
---     egroup_member table? 


--- Mapping between assemblies?
--- All default values need checking
--- Use MGED terms for design_types & experimental_variables where possible.
--- Use Brno nomelcature for target features



--
-- Table structure for table `egroup`
--

DROP TABLE IF EXISTS `egroup`;
CREATE TABLE `egroup` (
   `egroup_id` smallint(6) unsigned NOT NULL auto_increment,
   `name` varchar(40) NOT NULL default '',
   `location` varchar(120) default NULL,
   `contact` varchar(40) default NULL,
   PRIMARY KEY  (`egroup_id`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;


--insert into egroup values("", "efg", "Hinxton", "njohnson@ebi.ac.uk");

--- group is reserved by MySQL.
---Others? head? description? egroup_member(table?) probably overkill.


--- Need to separate fields between array and array_chip sensibly
--- It seems Nimblgen can do pretty much anything they like in a chip set, but can we make any safe assumption that will be array/chip set specific

--
-- Table structure for table `array`
--

DROP TABLE IF EXISTS `array`;
CREATE TABLE `array` (
   `array_id` int(10) unsigned NOT NULL auto_increment,
   `name` varchar(40) default NULL,
   `format` varchar(20) default NULL,
   `vendor` varchar(40) default NULL,
   `description` varchar(255) default NULL,
   `type` varchar(20) default NULL,
   PRIMARY KEY  (`array_id`),
   UNIQUE KEY   (`vendor`, `name`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;



-- removed `size` tinyint(4) unsigned NOT NULL default '0', now dynamically generated from key array_chips
--- Size -> Chips?  Need some level of validation for chip sets to avoid incomplete chip sets
--- name = design_name, or is this the chip name?
--- format = tiled, gene, exon, targetted, custom/mixed? Need control/restrict these etc..
--- species, could we have multi-species arrays?
--- removed:
--   `design_id` int(11) unsigned NOT NULL default '0',

--
-- Table structure for table `array_chip`
--

DROP TABLE IF EXISTS `array_chip`;
CREATE TABLE `array_chip` (
   `array_chip_id` int(10) unsigned NOT NULL auto_increment,
   `design_id` varchar(20) default NULL,
   `array_id` int(10) unsigned NOT NULL,
   `name` varchar(40) default NULL,
    PRIMARY KEY  (`array_chip_id`),
   UNIQUE KEY `array_design_idx` (`array_id`, `design_id`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;


--- name = design_name, or is this the chip name?
--- removed  `description` varchar(255) default NULL,




--
-- Table structure for table `probe_feature`
--

DROP TABLE IF EXISTS `probe_feature`;
CREATE TABLE `probe_feature` (
   `probe_feature_id` int(10) unsigned NOT NULL auto_increment,
   `seq_region_id` int(10) unsigned NOT NULL default '0',
   `seq_region_start` int(10) NOT NULL default '0',
   `seq_region_end` int(10) NOT NULL default '0',
   `seq_region_strand` tinyint(4) NOT NULL default '0', 
   `coord_system_id` int(10) unsigned NOT NULL default '0',
   `probe_id` int(10) unsigned NOT NULL default '0',
   `analysis_id` int(10) unsigned NOT NULL default '0',	
   `mismatches` tinyint(4) NOT NULL default '0',
   `cigar_line` text,
   PRIMARY KEY  (`probe_feature_id`),
   KEY `probe_idx` (`probe_id`),
   KEY `seq_region_idx` (`seq_region_id`, `seq_region_start`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;


-- remove mismatches?
--- Do jointindex on seq region local start? ADD COORD_SYSTEM to joint index?
--- Currently use chr data in here rather than true seq_regions, Need to map all probe global values to seq_regions.
--- build_id currently set to freeze date, can go when we incorporate build mapping into the import API
--- Would not capture true probe seq if there were any mismatches in mapping.
--- Mismatches in mapping, should default be NULL?  Mismatch for nimblegen here would denote MM of PM/MM probe pair
--- Cigarline(from alignment) shows mismatches, would normally be "probe.length"M, would need original probe seq for alignment, not currently stored with seq_region style tables.  See comments below probe table re: seq storage.
--- Joint index on seq_region_id and local start? 



--
-- Table structure for table `probe_set`
-- 

DROP TABLE IF EXISTS `probe_set`;
CREATE TABLE `probe_set` (
   `probe_set_id` int(10) unsigned NOT NULL auto_increment,
   `name` varchar(20) NOT NULL default '',
   `size` smallint(6) unsigned NOT NULL default '0',
   `family` varchar(20) default NULL,
   PRIMARY KEY  (`probe_set_id`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;

--- Now ancillary/optional table
--- joint key on probe_set_id and array_id?
--- aka feature for nimblegen(optional, therefore some probes = their featureset/probeset)
--- Can we omit probe_sets for sets of size == 1?
--- family= ENCODE REGIONS, RANDOM etc, generic descriptor for probes, can have multiple families on one chip
--- xref_id = encode region name, removed!!!!  Generate real xref entries.


---
-- Table structure for table `probe`
--

DROP TABLE IF EXISTS `probe`;
CREATE TABLE `probe` (
   `probe_id` int(10) unsigned NOT NULL auto_increment,
   `probe_set_id` int(10) unsigned default NULL,
   `name` varchar(40) NOT NULL default '',
   `length` smallint(6) unsigned NOT NULL default '0',
   `array_chip_id` int(10) unsigned NOT NULL default '0',
   `class` varchar(20) default NULL,
    PRIMARY KEY  (`probe_id`, `name`),
    KEY `array_chip_idx` (`array_chip_id`),
    KEY `probe_set_idx` (`probe_set_id`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;


--- Add X & Y coords
--- remove array_id and link through probe_set, can't if we're not populating probe_set for sets of 1.
--- name (for Affy) at least is array_chip to probe relationship, probeset is optional for some formats 
--- remove? class = control, experimental etc... naming clash with array.class, different class types.
--- pair_index aka nimblegen match_index, id to connect paired probes (same as id/name for single probes).
--- 
--- Still have issues will true probe seq storage?  Some probes have mismatches and map multiple times therefore cannot use genome seq?
--- Seq storage not necessary for core, but maybe for FG analysis DB?
--- Have seq field/table?  Huge amount of probes and theoretically some could be >>1000bps.  If probes never overlapped would actually be less seq data as probe should be non-redundant within array.  Not the case, but may be easiest way to maintain true probe seq data.  Would negate the need for seq_region tables, this may hinder remapping. Then just have mismatches and cigar_line in probe_feature table instead of in both.  Would then also need build and chromosome tables.  How important is this?

--- As nimblegen probes are premapped to one location, can we capture this in probe_feature.analysis_id (e.g. vendor_mapping)?  Then alter this when we remap.  How can we maintain true match/mismatch pairs on remapping?  Only map PM probe, and then place MM probe at same location.  So mismatch in probe_feature would denote PM or MM probe only if probe/pair_index is valid and, vice versa.


--- Xref issue:  How are we going to consolidate vendor defined xrefs vs. ensembl core xrefs (e.g. affy) to ensure updating of xref table in core DB?



--- pair_index table   `pair_index` int(10) unsigned NOT NULL default '0',
--- joint index on probe_ids

--- Other fields:
---     length?
--- Removed:
---       `seq` mediumtext NOT NULL, - now captured via probe_feature.seq_region_id, see above for seq storage issues.
---	  `mismatch` tinyint(4) NOT NULL default '0', - Now using probe_feature, see above
---       array_id, now in probe_set


---
-- Table structure for table `probe_design`
--

DROP TABLE IF EXISTS `probe_design`;
CREATE TABLE `probe_design` (
   `probe_id` int(10) unsigned NOT NULL default '0',
   `analysis_id` int(10) unsigned NOT NULL default '0',
   `score` double default NULL,	
   `coord_system_id` int(10) unsigned NOT NULL default '0',
    PRIMARY KEY  (`probe_id`, `analysis_id`, `coord_system_id`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;


---
--- really wanted default NULL for coord_system_id but cannot have with int

--
-- Table structure for table `experiment`
--

DROP TABLE IF EXISTS `experiment`;
CREATE TABLE `experiment` (
   `experiment_id` int(10) unsigned NOT NULL auto_increment,
   `name` varchar(30) default NULL,
   `egroup_id` smallint(6) unsigned default NULL,
   `date` date default '0000-00-00',
   `primary_design_type` varchar(30) default NULL, 
   `description`  varchar(255) default NULL,
   `mage_xml_id` int(10) unsigned default NULL,
   PRIMARY KEY  (`experiment_id`),
   UNIQUE KEY `name_idx` (`name`),
   KEY `egroup_idx` (`egroup_id`),
   KEY `design_idx` (`primary_design_type`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;


-- alter table experiment add schema_build?  This would enable trace back to original import DB, just incse seq_region_ids change, or use status? IMPORTED_schema_build, MAPPED_schema_build or cs_id?
-- remove primary design_type
--- design_type  = CHIP2 etc... (is also design type in ontology i.e. binding_site_identification)
--- Secondary design type may be redundant, so have associated_design_types table, containing MGED ontology types?  Or have ontology_types table to control input and have linker table with just IDs?  Too normalised?


--
-- Table structure for table `experimental_design`
--

DROP TABLE IF EXISTS `experimental_design`;
CREATE TABLE `experimental_design` (
   `design_type_id` int(10) unsigned NOT NULL auto_increment,
   `table_name` varchar(40) default NULL,
   `table_id` int(10) unsigned default NULL,	
   PRIMARY KEY  (`design_type_id`, `table_name`, `table_id`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;


--
-- Table structure for table `mage_xml`
--

DROP TABLE IF EXISTS `mage_xml`;
CREATE TABLE `mage_xml` (
   `mage_xml_id` int(10) unsigned NOT NULL auto_increment,
   `xml` text,
   PRIMARY KEY  (`mage_xml_id`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;






-- Do we need a key just on table_id and name?

-- Handles design_types other than the primary experimental.primary_design_type

--
-- Table structure for table `design_type`
--

DROP TABLE IF EXISTS `design_type`;
CREATE TABLE `design_type` (
   `design_type_id` int(10) unsigned NOT NULL auto_increment,
   `name` varchar(255) default NULL,
   PRIMARY KEY  (`design_type_id`),
   KEY `design_name_idx` (`name`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;


-- add description here?
-- Handles design_types other than the primary experimental.primary_design_type



-- Table structure for table `feature_type`

DROP TABLE IF EXISTS `feature_type`;
CREATE TABLE `feature_type` (
   `feature_type_id` int(10) unsigned NOT NULL auto_increment,
   `name` varchar(40) default NULL,
   `class` varchar(40) default NULL,
   `description`  varchar(255) default NULL,
   PRIMARY KEY  (`feature_type_id`),
   UNIQUE KEY `name_class_idx` (`name`, `class`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;


--- Table to contain Brno nomenclature (modification ontology?) etc.
--- enum on class? HISTONE, PROMOTER
--- Have ontology name in field?
--- Have load ontology tool script?


-- Table structure for table `data_set`

DROP TABLE IF EXISTS `data_set`;
CREATE TABLE `data_set` (
   `data_set_id` int(10) unsigned NOT NULL auto_increment,
   `result_set_id` int(10) unsigned default '0',
   `feature_set_id` int(10) unsigned default '0',
   `name` varchar(40) default NULL,
   PRIMARY KEY  (`data_set_id`, `result_set_id`, `feature_set_id`),
   UNIQUE KEY `name_idx` (name)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;


-- PRIMARY KEY is just to impose uniqueness
-- any more keys?  Primary access would be through result_set_id to test if it appears as displayable in status, but this will just be a straight list of all
-- so maybe we can put that at the end of the index and the most used for query at the start?


-- Link table to provide many to one relationships:
--	target/feature < experiment (via experimental_chip_id)
--	prediction < experiment (via feature_set.feature_set_id)
-- Having this keyed experimental_chip also allows us to have a feature spanning two chips from the same exxperiment, aswell a delineating between potentially differentially formated chips from the same experiment.  Don't really need this level of info, just experiment_id will suffice.
-- Can we just use the index and drop the table?  Not that big so doesn't matter
-- Omitting experimental_feature_type_id from feature_set also allows us to drop this table for lite?


--THis was originally also the record of the experiment feature_type, but this is now NR in this table.
-- We could move it to the result_set table, is more focused on the experimental_chips
-- But would potentially also be NR there too.  
-- Is still NR in result_set as we can have several analyses on the same chip/channels
-- Feature type is also known before results are imported on an exp/ec level, but would always import results anyway
-- So it's logically a little removed from where it should be, but more practical from a result set handling point of view.
-- could have in:
---- result_feature, but would not necessarily have feature_set_id
---- result_set
---- experimental_chip


-- Table structure for table `experiment_prediction`

--DROP TABLE IF EXISTS `experiment_prediction`;
--CREATE TABLE `experiment_prediction` (
--  `experiment_id` int(10) unsigned default NULL,
--   `predicted_feature_id` int(10) unsigned default NULL,
--   PRIMARY KEY  (`experiment_id`, `predicted_feature_id`)
--) ENGINE=MyISAM DEFAULT CHARSET=latin1;

-- Superceded by feature_set
-- Provides link between predicted_features and contributing experiments
-- Which may be a subset of those in experiment_target

-- Table structure for table `result`

DROP TABLE IF EXISTS `result`;
CREATE TABLE `result` (
   `result_id` int(10) unsigned NOT NULL auto_increment,
   `probe_id` int(10) unsigned default NULL,
   `score` double default NULL,
   `chip_channel_id` int(10) unsigned NOT NULL,
   `X` int(4) unsigned default NULL,
   `Y` int(4) unsigned default NULL,
   PRIMARY KEY  (`result_id`),
   KEY `probe_idx` (`probe_id`),
   KEY `chip_channel_idx` (`chip_channel_id`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1 MAX_ROWS=100000000  AVG_ROW_LENGTH=40;


--- joint index on all but result_id and maybe analysis_id?
--- REMOVEd experimental_id?   `experimental_chip_id` int(10) unsigned default NULL,
--- joint primary key with probe_feature_id?
--- metric default would be id for "RAW"..no, need to test has been specifically set, so NULL
--- Allows storage of none raw values
---Also needs to accommodate different normalisations 
---    `table_id` int(10) unsigned default NULL,
---   `table_name` varchar(20) default NULL,

--- Table structure for `result_set`

DROP TABLE IF EXISTS `result_set`;
CREATE TABLE `result_set` (
   `result_set_id` int(10) unsigned NOT NULL auto_increment,
   `analysis_id` int(10) unsigned default NULL,
   `name` varchar(40) default NULL,
   `cell_type_id` int(10) unsigned default NULL,
   `feature_type_id` int(10) unsigned default NULL,
   PRIMARY KEY  (`result_set_id`),
   UNIQUE KEY `unique_idx` (`name`,`analysis_id`,`feature_type_id`,`cell_type_id`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;

-- This is still v hard to impose unique constraints without key's of every combination of 2 :(
-- should we just go for NR table
-- chip_channel_id ID's a unique analysis instance of a physical chip or channel, used to key into results, needs to be autoinc really :(
-- Have in separate table 


--- WE STILL NEED A TABLE SET ID i.e. to differentiate results by chip, otherwise we cannot deal with result within a result_set on a chip by chip basis
--- table_set_id should be unique?
--- table_set_id table_name table_id and analysis_id shoudl also be unique
--- result_set_id and table_set_id should be unique

--- we could have another table here: table_set otherwise, table_name, analysis_id, result_set_id become redundant
--- table/chip_set: table_set_id, table_id, result_set_id

-- this is basically using the table_set_id to provide a link to the table_name to deconvolute the table_name/id relationship
-- we want one id in the result table to denote the chip/channel, analysis.




DROP TABLE IF EXISTS `chip_channel`;
CREATE TABLE `chip_channel` (
   `chip_channel_id` int(10) unsigned NOT NULL auto_increment,
   `result_set_id` int(10) unsigned default '0',
   `table_id` int(10) unsigned NOT NULL,
   `table_name` varchar(20) NOT NULL,
   PRIMARY KEY  (`chip_channel_id`, `result_set_id`),
   UNIQUE KEY `rset_table_idname_idx` (`result_set_id`, `table_id`, `table_name`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;

-- unique key is required? can we make the api check this to reduce size
-- unqiue key does not provide access from table_id/name side
-- should we just have table_name here, as it's more to do with the chip_channel than the result set
-- would introduce some redundancy, but low volume table so not a problem.

-- Do we need to remove result_set_id too, so we can have an expermental_set in it's own right iwthout the associated results?
-- We could then defined channel level relationship from these sets rather than storing these too
-- However we still need to store the channel results which will require a experimental_set_id
-- This currently is redundant with respect to physical sets due to the possibility of differeing analyses on the same raw data.

-- So two problems:
-- Need to represent channel->result relstionship, can we set all both chsnnels across multiple chips in set?
-- Need to represent single set, and potentially multiple instance of this set through differing analyses
-- Need to register these sets before we can record raw results. May not know which chips are in same set to begin with.

-- 3 solutions:
-- 1 Big redundant result_set table and hang the individual experimental_sets? Have unique table_set_id key'd to result
--- We would have to update result appropriately or insist on chip sets being known in advance
--- REDUNDANT
--- No access to chip sets without accessing previous result_set or same ec ids?  OR could have chip_set_id in experimental_chip
--- TABLE LIGHT
-- 2 Midway solution, Have experimental/analysis_set table which represent
--- LESS REDUNDANT
--- Same problems as above
-- 3 Have experimental_set to represent the physical unique set, also have analysis_set table to link between result_set and experimental_set
-- result would have analysis_set_id
--- Would have same probs as above but would haave uniquely accessible chip/experimental sets and NR all round.
--- NON_REDUNDANT
--- Access to chip sets
--- TABLE HEAVY
--- Would still have update problem

--- This is just replicating what we have in experiment or channel, too complicated, result_set will be low volume so just deal with redundancy
--- Can we still maintain unique result_set id and have a link result_table_entries, this will simply contain the individual table_id's, so call it chip_channel_result?


--- we could drop this table and just keep the index for e! prd


--- THe problem is only link features to chip sets, not individual chips, therefore result_group_id needs to be NR with respect to ec's within same set. Analysis_id delineates result_groups of the chip_set(tablename/id)
--- More keys?  Above key only defines uniqueness and linking from result_set, do we need exp > result_group query?

--- > 1 chip set i.e. duplicates are linked to one(or many) result sets(feature_sets)
--- but each can have a displayable status set, but only in it's own context i.e. you can have it displayed in one set and not in another.

--- Now we have problem of channel result sets:
---	chip set values still valid
---     would be nice to be able to extract chip_set to it's own table/experimental_chip
---     move chip_set_id to experimental_chip? result_group_id effectively denotes chip set, but needs to use ec.chip_set_id as reference


--
-- Table structure for table `predicted_feature`
--

DROP TABLE IF EXISTS `annotated_feature`;
CREATE TABLE `annotated_feature` (
  `annotated_feature_id` int(10) unsigned NOT NULL auto_increment,
  `seq_region_id` int(10) unsigned NOT NULL default '0',
  `seq_region_start` int(10) unsigned NOT NULL default '0',
  `seq_region_end` int(10) unsigned NOT NULL default '0',
  `seq_region_strand` tinyint(1) NOT NULL default '0',
  `coord_system_id` int(10) unsigned NOT NULL default '0',	
  `display_label` varchar(60) default NULL,
  `score` double default NULL,
  `feature_set_id` int(10) unsigned NOT NULL default '0',
  PRIMARY KEY  (`annotated_feature_id`),
  KEY `seq_region_idx` (`seq_region_id`,`seq_region_start`),
  KEY `feature_set_idx` (`feature_set_id`)	  
) ENGINE=MyISAM DEFAULT CHARSET=latin1 MAX_ROWS=100000000 AVG_ROW_LENGTH=80;

--- Need to be able to maintain link between prediction and source data i.e. experiment/s
--- Need to be able to have this table running completely from the persistent table
--- i.e. We want to be able to remove the experiment table data and still have access to all relevant data
--- e.g. primary design type, target_name/description 
--- Either make experiment->target 1:1 and have mixed persistent data in target(rm experiment_id and add target_id to experment)
--- Or add target_id to predicted_feature, may be redundant for non-tiling arrays?  How are we going to capture primary_design_type?


--- Table structure for `feature_set`

DROP TABLE IF EXISTS `feature_set`;
CREATE TABLE `feature_set` (
   `feature_set_id` int(10) unsigned NOT NULL auto_increment,
   `feature_type_id` int(10) unsigned NOT NULL,
   `analysis_id`  int(10) unsigned default NULL,
   `cell_type_id` int(10) unsigned default NULL,
   `name` varchar(250) default NULL,
   PRIMARY KEY  (`feature_set_id`),
   KEY `feature_type_idx` (`feature_type_id`),
   UNIQUE KEY `name_idx` (name)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;


-- make analysis NOT NULL
-- change default NOT/NULLs?
-- feature_type_id is NR between this and pf, but need it here for ft type ResultSet queries 



--
-- Table structure for table `experimental_chip`
--

DROP TABLE IF EXISTS `experimental_chip`;
CREATE TABLE `experimental_chip` (
   `experimental_chip_id` int(10) unsigned NOT NULL auto_increment,
   `unique_id` varchar(20) NOT NULL default '0',
   `experiment_id` int(10) unsigned default NULL,
   `array_chip_id` int(10) unsigned default NULL,
   `feature_type_id` int(10) unsigned default NULL,
   `cell_type_id` int(10) unsigned default NULL,
   `biological_replicate` varchar(40) default NULL,
   `technical_replicate` varchar(40) default NULL,
   PRIMARY KEY  (`experimental_chip_id`),
   KEY `experiment_idx` (`experiment_id`),
   KEY `feature_type_idx` (`feature_type_id`),
   KEY `unique_id_idx` (`unique_id`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;

-- We can't implement uniqueness of ec(unique_id for a given vendor) via key here
-- There may be clashes between vendors, and a chip could potentially be re-used in another exp
-- This is handled by the fetch methods:
--    fetch_by_unique_and_experiment_id
--    fetch_by_unique_id_vendor
-- add key on cell_type? or remove key on feature_type.  Will we ever want to query based on these? I doubt it.
--Should handle re-usage of physical chip


--
-- Table structure for table `channel`
--

DROP TABLE IF EXISTS `channel`;
CREATE TABLE `channel` (
   `channel_id` int(10) unsigned NOT NULL auto_increment,
   `experimental_chip_id` int(10) unsigned default NULL,	
   `sample_id` varchar(20) default NULL,
   `dye`  varchar(20) default NULL,
   `type` varchar(20) default NULL,
   PRIMARY KEY  (`channel_id`),
   KEY `experimental_chip_idx` (`experimental_chip_id`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;

-- type should be restricted to EXPERIMENTAL & CONTROL?
-- all other variables should be in experimental_variable table

--
-- Table structure for table `cell_type`
--

DROP TABLE IF EXISTS `cell_type`;
CREATE TABLE `cell_type` (
   `cell_type_id` int(10) unsigned NOT NULL auto_increment,
   `name`  varchar(120) not NULL,
   `display_label` varchar(20) default NULL,
   `description` varchar(40) default NULL,
   PRIMARY KEY  (`cell_type_id`),
   UNIQUE KEY `name_idx` (`name`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;

-- type? enum Tissue? Line?
-- xref to coriell?

--
-- Table structure for table `experimental_variable`
--

DROP TABLE IF EXISTS `experimental_variable`;
CREATE TABLE `experimental_variable` (
   `table_id` int(10) unsigned default NULL,
   `table_name` varchar(20) default NULL,	
   `name` varchar(40) default NULL,
   `unit` varchar(40) default NULL,
   `value` varchar(40) default NULL,
   PRIMARY KEY  (`table_id`, `table_name`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;


-- context_id = table_name:table_name_id
-- Needs to accommodate QC flag and description, and reusing chips.
-- Have list of mandatory defs for each group, then also test MGED ontology and warn if not present.
-- status for chip?
-- Some of these are chip rather than channels specific
-- `tissue` varchar(40) NOT NULL default '',
-- `cell_type` varchar(40) NOT NULL default '',
-- `dev_stage` varchar(40) NOT NULL default '',
-- `antibody` varchar(40) NOT NULL default '',
-- `time_point` time NOT NULL default '000:00:00',
-- antibody?
-- species
-- description?
--- can this handle experimental management such that slides can be reused and classed as such
--- Or classed as failures etc.
--- Do we also need an extra table to hold experiment level meta data along side design type, and rename this channel variable

--
-- Table structure for table `status`
--

DROP TABLE IF EXISTS `status`;
CREATE TABLE `status` (
   `table_id` int(10) unsigned default NULL,
   `table_name` varchar(20) default NULL,	
   `status_name_id` int(10) NOT NULL,
   PRIMARY KEY  (`table_id`, `table_name`, `status_name_id`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;


--
-- Table structure for table `status`
--

DROP TABLE IF EXISTS `status_name`;
CREATE TABLE `status_name` (
   `status_name_id` int(10) unsigned NOT NULL auto_increment,
   `name` varchar(20) default NULL,	
   PRIMARY KEY  (`status_name_id`),
   UNIQUE KEY `status_name_idx` (`name`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;


INSERT into status_name values ('', 'DISPLAYABLE');
INSERT into status_name values ('', 'IMPORTED');

--change to small int?
--INSERT into table status("", 'DISPLAYABLE');
-- need to add more states, probably need to validate/insert required states in Importer
-- would need to get CoordSys objects and set IMPORTED_CS_"cs_id" for relevant data_version



--
-- Table structure for table `analysis`
--

DROP TABLE IF EXISTS `analysis`;
CREATE TABLE `analysis` (
  `analysis_id` int(10) unsigned NOT NULL auto_increment,
  `created` datetime NOT NULL default '0000-00-00 00:00:00',
  `logic_name` varchar(40) NOT NULL default '',
  `db` varchar(120) default NULL,
  `db_version` varchar(40) default NULL,
  `db_file` varchar(120) default NULL,
  `program` varchar(80) default NULL,
  `program_version` varchar(40) default NULL,
  `program_file` varchar(80) default NULL,
  `parameters` varchar(255) default NULL,
  `module` varchar(80) default NULL,
  `module_version` varchar(40) default NULL,
  `gff_source` varchar(40) default NULL,
  `gff_feature` varchar(40) default NULL,
  PRIMARY KEY  (`analysis_id`),
  UNIQUE KEY `logic_name` (`logic_name`),
  KEY `logic_name_idx` (`logic_name`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;

--
-- Table structure for table `analysis_description`
--

DROP TABLE IF EXISTS `analysis_description`;
CREATE TABLE `analysis_description` (
  `analysis_id` int(10) unsigned NOT NULL default '0',
  `description` text,
  `display_label` varchar(255) default NULL,
  `displayable` tinyint(1) NOT NULL default '1',
  `web_data` text,	
  KEY `analysis_idx` (`analysis_id`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;


--
-- Table structure for table `meta`
--

DROP TABLE IF EXISTS `meta`;
CREATE TABLE `meta` (
  `meta_id` int(10) NOT NULL auto_increment,
  `meta_key` varchar(40) NOT NULL default '',
  `meta_value` varchar(255) NOT NULL default '',
  PRIMARY KEY  (`meta_id`),
  KEY `meta_key_index` (`meta_key`),
  KEY `meta_value_index` (`meta_value`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;


--
-- Table structure for table `meta_coord`
--

DROP TABLE IF EXISTS `meta_coord`;
CREATE TABLE `meta_coord` (
  `table_name` varchar(40) NOT NULL default '',
  `coord_system_id` int(10) NOT NULL default '0',
  `max_length` int(11) default NULL,
  UNIQUE KEY `table_name` (`table_name`,`coord_system_id`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;

--- change to primary key?
---should only ever be predicted_feature, but with all the coord_sys_ids
---This is slightly redundant, but required for core API modules to work

--- Set up default meta coord entries...this should be done in import
-- Change max lenght of oligo?

insert into meta_coord values("annotated_feature", 1, 5000);
insert into meta_coord values("annotated_feature", 1, 2000);

--
-- Table structure for table `coord_system`
--

DROP TABLE IF EXISTS `coord_system`;
CREATE TABLE `coord_system` (
  `coord_system_id` int(10) NOT NULL auto_increment,
  `name` varchar(40) NOT NULL default '',
  `version` varchar(40) default NULL,
  `rank` int(11) NOT NULL default '0',
  `attrib` set('default_version','sequence_level') default NULL,
  `schema_build` varchar(10) default NULL,
  `core_coord_system_id` int(10) NOT NULL, 
  PRIMARY KEY  (`coord_system_id`, `core_coord_system_id`, `schema_build`),
  KEY `name_version_idx` (`name`, `version`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;



--- This is never being queried anyway as we cache all the CSs on start up!
--- UNIQUE KEY `rank` (`rank`, `schema_build`),
--- UNIQUE KEY `name` (`name`,`version`, `schema_build`)
--- we want nr coord_system_id records to accomodate multiple coord_sys's which are effectively the same
--- e.g. NCBI36 chromosome across all the core DB which have it
--- can we ignore the unique keys for rank, name & version as these will be implied by the core DB?
--- primary key should really be coord_sys_id, name, version, schema_build (core_coord_sys_id implied by other values)
--- This however gives no access to the rest of the index for most of the queries
--- don't need to really bother optimising the key structures as the table is so small?
--- put core_coord_system_id at end of primary key and have separate key for coord_system_id
--- primary key name, schema_build, version, core_coord_system_id
--- or could we jsut depend on the core keys confering data integrity and have key optimised for query


--
-- Table structure for table `seq_region`
--

DROP TABLE IF EXISTS `seq_region`;
CREATE TABLE `seq_region` (
  `seq_region_id` int(10) unsigned NOT NULL auto_increment,
  `name` varchar(40) NOT NULL default '',
  `coord_system_id` int(10) unsigned NOT NULL default '0',
  `core_seq_region_id` int(10) unsigned NOT NULL default '0',
  `schema_build` varchar(10) default NULL,
  PRIMARY KEY  (`seq_region_id`, `name`, `schema_build`),
  KEY `coord_system_id` (`coord_system_id`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1; 





-- it maybe possible to have 2 seq_regions on different levels with the same name
-- there fore have to use core cs id in primary key
-- order of extra primary key members doesn't really matter as we'll never query on them?
-- swapped order of ke to name coord_system_id, as we will probably want to query on name primarily
-- why does core table have cs id key?

-- Name is only required to enable us to add new seq_regions to the correct seq_region_id
-- It will never be used to retrieve a slice as we do that via the core DB
-- how are we going to use this when querying?
-- basically pull back seq_region_id based schema_build and core_seq_region_id
-- can we omit core_coord_system_id? As we have this info from the cs table.
-- other keys?


--- Further thoughts:

--- More tables required for probe remapping? meta, rule tables etc?


--- Denormalise ---
--- To mart style to optimize queries? Mart style interface to export(to R)?
--- probe_set, probe, probe_feature and results could all e split on array vendor, format or experiment.design_type
---     probe > "array.vendor"_probe e.g. affy_probe, or "array.name"_probe e.g u133_probe, or "array.class"_probe e.g. CHIP2_probe


--- Mostly empty/unused fields which could be extracted to separate tables to reduce size?
---     cigar_line
